function [equalflag, type, info] = arrcompare(X,Y) 
# usage: [equalflag, type, info] = arrcompare(X,Y) 
#
# This function compares two multidimensional arrays "X" and "Y".
# The "equalflag" is set equal to one if and only if
# 1.  The dimension of both arrays is equal (type=1).
# 2.  The index ranges of both arrays are equal (type=2).
# 3.  All the elements in both arrays at the same locations are equal (type=3).
# Otherwise, "equalflag" is set equal to zero.  
#
# If the optional "type" variable is listed in the output arguments,  
# its value is set to the equality criterion number which is violated 
# first (the criteria are checked in the order shown).
#
# If the optional "info" variable is listed in the output arguments, 
# then for each type of inequality:
# 1.  "info" is the difference in dimension between the arrrays
# 2.  "info" is the boolean comparison vector of the index ranges
#     (e.g., ([2 3]==[2 4])=[1 0])
# 3.  "info" lists the indices of the elements which are unequal.
# Note that a full listing of all the unequal elements is only 
# computed when the "info" parameter is requested.  Otherwise,
# the script returns when the first unequal elements are found. 

# Author:  Ronald Kumon

  # Error checking
  if ( nargin != 2 )
    usage("arrcompare(X,Y)");
  endif

  if ( ( is_struct(X) != 1 ) || ( is_struct(Y) != 1 ) )
    error("At least one input arguments is a not a multidimensional array structure.");
  end

  # Default values
  equalflag=1;
  type=0;

  if ( X.ndim != Y.ndim) 
    equalflag=0;
    type=1;
    info=abs(X.ndim-Y.ndim);
    return
  end

  equaldims=(X.dims==Y.dims);
  if ( all(equaldims) != 1 ) 
    equalflag=0;
    type=2;
    info=equaldims;
    return
  end

  for arrindex=1:prod(X.dims)
    if ( X.elems(arrindex) != Y.elems(arrindex) )
      equalflag=0;
      type=3;
    end
  end

  if ( equalflag==1 )
    info=0;
    return
  end

  if ( type==3 ) 
    if ( nargout < 3 ) 
      return
    else
      count=1;
      for arrindex=1:prod(X.dims)
        if ( X.elems(arrindex) != Y.elems(arrindex) )
          info(count,:)=arrindex2coord(arrindex,X.dims).';
          count=count+1;
        end
      end
      return
    end
  end


endfunction

