function Z = arrcontr(X,Xvarindex,Y,Yvarindex,varindexperm) 
# usage: Z = arrcontr(X,Xvarindex,Y,Yvarindex,[varindexperm])   
#
# This function takes two multidimensional arrays "X" and "Y" and 
# contracts over a set of indices by applying the summation convention 
# with the specified indices "Xvarindex" of "X" and "Yvarindex" of "Y".
# "Xvarindex" and "Yvarindex" can be vectors which describe the 
# multiple indices which are to summed over.  The order of the variable
# indices of "X" relative to the variable indices of "Y" is given 
# by "varindexperm" in standard cyclic permutation form.
# If "varindexperm" is ommitted from the argument list, then 
# it is assumed that contraction is only occuring over one index.
# The results are put into a new multidimensional array "Z".  
#
# Uses:  arrindexorder.m, arrindex2coord.m, arrcoord2index.m 

# Author:  Ronald Kumon

  # Error checking
  if ( (nargin != 4) && (nargin != 5) )
    usage("arrcontr(X,Xvarindex,Y,Yvarindex,[varindexperm])");
  endif

  if ( ( is_struct(X) != 1 ) ||  ( is_struct(Y) != 1 ) )
    error("Input argument is not a multidimensional array structure.")
  end

  if ( ( is_vector(Xvarindex) != 1 ) || ( is_vector(Yvarindex) != 1 ) )
    error("Specified variable indices are not in vector form.")
  end

  if ( ( X.ndim== 0 ) || ( Y.ndim== 0 ) )
    error("Arrays cannot be contracted with scalars.  Use arrscalmult(a,X).")
  end

  if ( nargin == 5 )
    if ( is_vector(varindexperm) != 1 )
      error("Specified index permutation is not in vector form.")
    end
  end

  numXvarindex=length(Xvarindex);
  numYvarindex=length(Yvarindex);
  if ( numXvarindex != numYvarindex )
    error("Number of variable indices must be the same in both arrays.")
  end

  for index=1:numXvarindex 
    if ( ( Xvarindex(index) > X.ndim ) || ( Xvarindex(index) < 1 ) )
      error("First array: Specified variable index(es) are not in range of matrix dimensions.")
    end
  end

  for index=1:numYvarindex 
    if ( ( Yvarindex(index) > Y.ndim ) || ( Yvarindex(index) < 1 ) )
      error("Second array: Specified variable index(es) are not in range of matrix dimensions.")
    end
  end

  if ( nargin == 4 )
    if ( is_scalar(Xvarindex)==0 || is_scalar(Yvarindex)==0 )
      error("With four arguments, variable index must be a scalar.")
    end
    for index=1:numXvarindex
      if ( X.dims(Xvarindex(index)) != Y.dims(Yvarindex(index)) )
        error("At least one pair of variable indices do not have the same dimension.")
      end
    end
  end

  if ( nargin == 5 )
    if ( length(varindexperm) != numXvarindex )
      error("Length of permutation vector does not match number of variable indices.")
    end
    Yvarindexperm=Yvarindex(varindexperm);
    for index=1:numXvarindex
      if ( X.dims(Xvarindex(index)) != Y.dims(Yvarindexperm(index)) )
        error("At least one pair of variable indices do not have the same dimension.")
      end
    end
  end

  # Identify fixed indices of X and Y
  empty_list_elements_ok=1;  
  Xfixindex=[];
  for index=1:X.ndim
    if ( all(Xvarindex-index)==1 )    
      Xfixindex=[Xfixindex index];
    end
  end
  #Xfixindex
  Yfixindex=[];
  for index=1:Y.ndim
    if ( all(Yvarindex-index)==1 )    
      Yfixindex=[Yfixindex index];
    end
  end
  #Yfixindex
  empty_list_elements_ok=0;

  # Identify locations of fixed indices of X and Y
  # In a row vector with length equal to the number of dimensions, the
  # fixed indices are marked by ones, variable indices by zeros.
  Xfixindexflags=ones(1,X.ndim);
  for index=1:X.ndim 
    if ( all(Xvarindex-index)==0 )    
      Xfixindexflags(index)=0;
    end
  end
  Yfixindexflags=ones(1,Y.ndim);
  for index=1:Y.ndim 
    if ( all(Yvarindex-index)==0 )    
      Yfixindexflags(index)=0;
    end
  end
  #Xfixindexflags
  #Yfixindexflags

  # Compute number of dimensions of Z
  Z.ndim=X.ndim+Y.ndim-numXvarindex-numYvarindex;

  # Compute dimensions of Z  
  if ( Z.ndim == 0 )
    Z.dims=1;
  elseif ( isempty(Xfixindex) )
    Z.dims=Y.dims(Yfixindex);
  elseif ( isempty(Yfixindex) )
    Z.dims=X.dims(Xfixindex);
  else
    Z.dims=[X.dims(Xfixindex); Y.dims(Yfixindex)];
  end

  # Initialize elements for Z
  Z.elems=zeros(prod(Z.dims),1);
  
  for arrindex=1:prod(Z.dims)
    #arrindex
    # Find fixed indices of Z
    Zcoord=arrindex2coord(arrindex,Z.dims);
    Zcoordcount=1;
    numaddends=prod(X.dims(Xvarindex));
    for index=1:X.ndim
      if ( Xfixindexflags(index)==1 )
        Xindexmat(:,index)=Zcoord(Zcoordcount)*ones(numaddends,1);
        Zcoordcount=Zcoordcount+1;
      end
    end
    for index=1:Y.ndim
      if ( Yfixindexflags(index)==1 )
        Yindexmat(:,index)=Zcoord(Zcoordcount)*ones(numaddends,1);
        Zcoordcount=Zcoordcount+1;
      end
    end
    #printf("Assign variable indices:\n");
    Xvarindexmat=arrindexorder(X.dims(Xvarindex));
    Yvarindexmat=arrindexorder(Y.dims(Yvarindex));
    if ( nargin==5 )
      #printf("Permute variable indices:\n");
      Yvarindexmat=Yvarindexmat(:,varindexperm);
    end
    Xindexmat(:,Xvarindex)=Xvarindexmat;
    Yindexmat(:,Yvarindex)=Yvarindexmat;      
    # Find the corresponding coordinates for each addend of the sum    
    for index=1:numaddends  
      Xarrindex=arrcoord2index(Xindexmat(index,:),X.dims);
      Yarrindex=arrcoord2index(Yindexmat(index,:),Y.dims);
      #X.elems(Xarrindex);
      #Y.elems(Yarrindex);
      Z.elems(arrindex)=Z.elems(arrindex)+X.elems(Xarrindex)*Y.elems(Yarrindex);
    end
  end

endfunction

