function Z = arrsubarr(X,...)
# usage: Z = arrsubarr(X,...)  
#
# This function constructs the subarray of the multidimensional array "X"
# based on the ranges of indices specified by the extra arguments.
# For each dimension, the range of indices is indicated by inputting 
# a vector describing those indices (e.g., "startvalue:stepvalue:endvalue"
# or the magic-colon ":" for the whole range of indices).   
# If an index range is omitted, it is assumed to be one.
# As such, individual elements can be extracted by simply giving their indices.
# The size of the subarray is made as small as possible.
# The result is put into the new multidimensional array "Z".
#
# Uses:  arrforloops.m, arrcoord2index.m

# Author:  Ronald Kumon

  # Suppress warnings  
  empty_list_elements_ok=1;

  # Error checking
  if ( nargin < 1 )
    usage("arrsubarr(X,...)");
  endif

  if ( is_struct(X) != 1 )
    error("Input argument is not a multidimensional array structure.")
  end

  Xndim=X.ndim;
  Xdims=X.dims;

  if ( Xndim == 0 )
    error("It is not possible to take the subarray of a scalar.")
  end

  if ( nargin > Xndim+1 )
    error("The number of specified index ranges exceeds the array dimensions.")
  end  

  scalardir=0;
  for index=1:(nargin-1)
    vector=(1:Xdims(index))';
    range=vector(va_arg);
    #subvector(1:length(range),index)=range;
    subarrfirst(index)=range(1);
    subarrlast(index)=range(length(range));
    if ( is_scalar(range) )
      subarrsteps(index)=1;
      scalardir=scalardir+1;
    else
      subarrsteps(index)=range(2)-range(1);
    end
  end

  # If number of specified ranges is less than dimensions of X
  # assume each omitted range is 1.   This is done for partial 
  # MATLAB compatibility.  Unlike MATLAB, this routine will give
  # an error if the specified index range of the subarray exceeds the 
  # index range of the full array (though this seems sensible).
  if ( Xndim > nargin-1 )
    #printf("Not full\n");
    for index=nargin:Xndim
      #subvector(1,index)=1;
      subarrfirst(index)=1;
      subarrlast(index)=1;
      subarrsteps(index)=1;
      scalardir=scalardir+1;
    end
  end

  #subarrfirst
  #subarrlast
  #subarrsteps
  subarrdim=floor(subarrlast-subarrfirst)./subarrsteps+1;

  # Number of dimensions in subarray (minimum)
  Z.ndim=Xndim-scalardir;

  # Maximum index for each dimension 
  Zdimsfull=subarrdim;

  # Maximum index for each dimension with
  # dimensions stripped out with range of one
  if ( prod(subarrdim)==1 )
    Zdims=1;
  else
    Zdims=[];
    for index=1:Xndim
      if ( subarrdim(index) != 1 ) 
        Zdims=[Zdims; subarrdim(index)];
      end
    end
  end
  Z.dims=Zdims;
  
  # Compute coordinates of subarray elements from range vectors
  # Note that "arrforloop" a recursive routine which allows an 
  # arbitrary number of nested for loops to be generated
  # This is needed because the number of dimensions of the input
  # array is not known in advance.  The coordinate indices of the 
  # subarray elements are stored in "Xcoordsubarr".
  loops=flipud(Zdimsfull);
  startvector=subarrfirst.'-subarrsteps.';
  stepvector=subarrsteps.';
  count=1;
  Xcoordsubarr=[];
  [Xcoordsubarr,count]=arrforloops(Xndim,loops,startvector,stepvector,Xcoordsubarr,count);

  # Find subarray elements in X and copy them to Z
  Z.elems=zeros(prod(Z.dims),1);  
  for arrindex=1:prod(Z.dims)
    Xarrindexsubarr=arrcoord2index(Xcoordsubarr(arrindex,:),Xdims);
    Z.elems(arrindex)=X.elems(Xarrindexsubarr);
  end

  # Resume warnings   
  empty_list_elements_ok=0;

endfunction

