function Z = arrsubarrmatrix(X,rangematrix)
# usage: Z = arrsubarrmatrix(X,rangematrix)  
#
# This function constructs the subarray of the multidimensional array "X"
# based on the ranges of indices specified by the extra arguments.
# For each dimension, the range of indices is indicated by inputting 
# a matrix describing those indices in the form
#       (index_1)  [startvalue stepvalue endvalue;
#       (index_2)   startvalue stepvalue endvalue;
#                        ...
#       (index_n)   startvalue stepvalue endvalue]
# The size of the subarray is made as small as possible.
# The result is put into the new multidimensional array "Z".
#
# Uses:  arrforloops.m, arrcoord2index.m

# Author:  Ronald Kumon

  # Suppress warnings  
  empty_list_elements_ok=1;

  # Error checking
  if ( nargin < 1 )
    usage("arrsubarrmatrix(X,rangematrix)");
  endif

  if ( is_struct(X) != 1 )
    error("Input argument is not a multidimensional array structure.")
  end

  Xndim=X.ndim;
  Xdims=X.dims;

  if ( Xndim == 0 )
    error("It is not possible to take the subarray of a scalar.")
  end

  if ( is_matrix(rangematrix) != 1 )
    error("Input range matrix is not a matrix.")
  end

  if ( columns(rangematrix) != 3 )
    error("Range matrix must have three columns for start, step, and end index values.")
  end

  if ( rows(rangematrix) != Xndim )
    error("Range matrix must have one row for each dimension of the given array.")
  end

  subarrfirst=rangematrix(:,1);
  subarrsteps=rangematrix(:,2);
  subarrlast=rangematrix(:,3);

  scalardir=0;
  for index=1:rows(rangematrix)
    if ( is_scalar(subarrfirst(index):subarrsteps(index):subarrlast(index)) )
       scalardir=scalardir+1;
    end
  end

  subarrdim=floor(subarrlast-subarrfirst)./subarrsteps+1;

  # Number of dimensions in subarray (minimum)
  Z.ndim=Xndim-scalardir;

  # Maximum index for each dimension 
  Zdimsfull=subarrdim;

  # Maximum index for each dimension with
  # dimensions stripped out with range of one
  if ( prod(subarrdim)==1 )
    Zdims=1;
  else
    Zdims=[];
    for index=1:Xndim
      if ( subarrdim(index) != 1 ) 
        Zdims=[Zdims; subarrdim(index)];
      end
    end
  end
  Z.dims=Zdims;
  
  # Compute coordinates of subarray elements from range vectors
  # Note that "arrforloop" a recursive routine which allows an 
  # arbitrary number of nested for loops to be generated
  # This is needed because the number of dimensions of the input
  # array is not known in advance.  The coordinate indices of the 
  # subarray elements are stored in "Xcoordsubarr".
  loops=flipud(Zdimsfull);
  startvector=subarrfirst.'-subarrsteps.';
  stepvector=subarrsteps.';
  count=1;
  Xcoordsubarr=[];
  [Xcoordsubarr,count]=arrforloops(Xndim,loops,startvector,stepvector,Xcoordsubarr,count);

  # Find subarray elements in X and copy them to Z
  Z.elems=zeros(prod(Z.dims),1);  
  for arrindex=1:prod(Z.dims)
    Xarrindexsubarr=arrcoord2index(Xcoordsubarr(arrindex,:),Xdims);
    Z.elems(arrindex)=X.elems(Xarrindexsubarr);
  end

  # Resume warnings   
  empty_list_elements_ok=0;

endfunction

