function Z = arrsubassg(Xsub,X,...)
# usage: Z = arrsubassg(Xsub,X,...)  
#
# This function assigns a subarray "Xsub" to a equal or larger multidimensional
# array "X" based on the ranges of indices specified by the extra arguments.
# For each dimension, the range of indices is indicated by inputting 
# a vector describing those indices (e.g., "startvalue:stepvalue:endvalue"
# or the magic-colon ":" for the whole range of indices).   
# As such, individual elements can be extracted by simply giving their indices.
# The result is put into the new multidimensional array "Z".
#
# Uses:  arrforloops.m, arrcoord2index.m

# Author:  Ronald Kumon

  # Suppress warnings  
  empty_list_elements_ok=1;

  # Error checking
  if ( nargin < 2 )
    usage("arrsubassg(Xsub,X,...)");
  endif

  if ( is_struct(Xsub) != 1 )
    error("Input argument is not a multidimensional array structure.")
  end

  if ( is_struct(X) != 1 )
    error("Input argument is not a multidimensional array structure.")
  end

  Xsubndim=Xsub.ndim;
  Xsubdims=Xsub.dims
  Xndim=X.ndim;
  Xdims=X.dims

  if ( nargin-2 > Xndim )
    error("The number of specified index ranges exceeds the array dimensions.")
  end  

  for index=1:Xsubndim
    if ( Xsubdims(index) > Xdims(index) )
      error("An index range of the subarray exceeds the index range of the full array.")
    end
  end

  scalardir=0;
  for index=1:(nargin-2)
    vector=(1:Xdims(index))';
    range=vector(va_arg);
    subvector(1:length(range),index)=range;
    subarrfirst(index)=range(1);
    subarrlast(index)=range(length(range));
    if ( is_scalar(range) )
      subarrsteps(index)=1;
      scalardir=scalardir+1;
    else
      subarrsteps(index)=range(2)-range(1);
    end
  end

  # If number of specified ranges is less than dimensions of X
  # assume each omitted range is 1.  This is done for partial 
  # MATLAB compatibility.  Unlike MATLAB, this routine will give
  # an error if the specified index range of the subarray exceeds the 
  # index range of the full array (though this seems sensible).
  if ( Xndim > nargin-2 )
    printf("Not full\n");
    for index=nargin-1:Xndim
      subvector(1,index)=1;
      subarrfirst(index)=1;
      subarrlast(index)=1;
      subarrsteps(index)=1;
      scalardir=scalardir+1;
    end
  end

  #subarrfirst
  #subarrlast
  #subarrsteps
  subarrdim=floor(subarrlast-subarrfirst)./subarrsteps+1;

  # Number of dimensions in subarray (minimum)
  rangendim=Xndim-scalardir

  if ( rangendim != Xsubndim )
    error("Array size specified by the index ranges does not match size of given subarray.")
  end

  # Maximum index for each dimension 
  rangedimsfull=subarrdim

  # Maximum index for each dimension with
  # dimensions stripped out with range of one
  if ( prod(subarrdim)==1 )
    rangedims=1;
  else
    rangedims=[];
    for index=1:Xndim
      if ( subarrdim(index) != 1 ) 
        rangedims=[rangedims; subarrdim(index)];
      end
    end
  end
  rangedims
 
  # Compute coordinates of subarray elements from range vectors
  # Note that "arrforloop" a recursive routine which allows an 
  # arbitrary number of nested for loops to be generated
  # This is needed because the number of dimensions of the input
  # array is not known in advance.  The coordinate indices of the 
  # subarray elements are stored in "Xcoordsubarr".
  loops=flipud(rangedimsfull);
  startvector=subarrfirst.'-subarrsteps.';
  stepvector=subarrsteps.';
  count=1;
  Xcoordsubarr=[];
  [Xcoordsubarr,count]=arrforloops(Xndim,loops,startvector,stepvector,Xcoordsubarr,count)

  # Dimensions of new array are same as old array
  Z.ndim=X.ndim
  Z.dims=X.dims

  # Duplicate X into Z and then copy subarray elements in Xsub to Z
  Z.elems=X.elems;  
  for arrindex=1:prod(Xsubdims)
    Xarrindex=arrcoord2index(Xcoordsubarr(arrindex,:),Xdims);
    Z.elems(Xarrindex)=Xsub.elems(arrindex);
  end
  Z.elems

  # Resume warnings   
  empty_list_elements_ok=0;

endfunction

