function Z = arrreg2voigttwo(X) 
# usage: Z = arrreg2voigttwo(X) 
#
# This function takes a 2n-dimensional array labelled in regular notation 
# and generates the corresponding n-dimensional array in Voigt notation.
# As is conventional for certain tensors (e.g., strain, compliance), 
# a factor of two is included for every index which is equal to 
# 4, 5, or 6.
# [see B. A. Auld, Acoustic Fields and Waves in Solids (John Wiley 
# & Sons, New York, 1973), Sections 1F and 3C.]
#
# Uses:  arrindex2coord.m, arrcoord2index.m, arrvoigtmapinv.m

# Author:  Ronald Kumon

  # Error checking
  if ( nargin != 1 )
    usage("arrarr2voigttwo(X)");
  endif

  if ( is_struct(X) != 1 ) 
    error("The input argument is a not a multidimensional array structure.");
  end

  if ( is_tensor(X) != 1 ) 
    error("The input array is not a tensor.");
  end

  if ( rem(X.ndim,2) != 0)
    error("The input array does not have an even number of dimensions.")
  end

  if ( X.dims(1) != 3 ) 
    error("The input array is not in appropriate form (three indices in each dimension).");
  end

  warning("Regular to Voigt index conversion with factors of two used.")

  # Number of dimensions in full array
  Z.ndim=X.ndim/2;

  # Index ranges in full array 
  Z.dims=6*ones(Z.ndim,1);

  # Elements of full array
  for arrindex=1:prod(Z.dims)
    factor=1;
    Zcoordindiv=arrindex2coord(arrindex,Z.dims);
    for index=1:Z.ndim
      Xvoigtindex(arrindex,(2*index-1):(2*index))=arrvoigtmapinv(Zcoordindiv(index));
      if ( Zcoordindiv(index) > 3 )
        factor=factor*2;
      end
    end
    Xindex=arrcoord2index(Xvoigtindex(arrindex,:),X.dims);
    Z.elems(arrindex)=X.elems(Xindex)*factor;
  end

endfunction

